;;;
;;; List All Buffers ...
;;;
;;; This file is a almost dead copy of Flist_buffers in buffer.c .
;;; Converted to emacs lisp and added some features by enami@ptgd.sony.co.jp
;;;
;;; $Id: list-all-buffers.el,v 1.3 90/02/21 08:03:32 enami Exp $
;;;
(provide 'list-all-buffers)

;;;
;;; There is 2 way to access buffer's local variables.
;;; Which way is better ?
;;; This way, and ...
;;;
(defconst buffer-attribute-list
  '(buffer-file-name buffer-read-only (buffer-size) mode-name)
  "Current gettable buffer attribute.")

(defun buffer-attribute (&optional buffer)
  "Return BUFFER's attribute as a list.  Currently available elements are:
 0. buffer-file-name
 1. buffer-read-only
 2. buffer-size
 3. mode-name

If buffer is not exist, NIL is returned.  Argument BUFFER is optional.  If
it is omitted, current buffer is used  (but it is almost non-sense :-)"
  (setq buffer (or buffer (current-buffer)))
  (save-excursion
    (set-buffer buffer)
    (mapcar 'eval buffer-attribute-list) ))

(let* (e (i 0) (l buffer-attribute-list))
  (while l
    (setq e (car l))
    (put (if (consp e)
	     (car e)			;function name
	   e)				;variable name
	 'buffer-attribute i)
    (setq i (1+ i)
	  l (cdr l))))

(defun access-buffer-attribute (attr val)
  "Accessor buffer's attribute. ATTR must be one element of
`buffer-attribute-list' and VAL must be value returned from buffer-attribute."
  (nth (get val 'buffer-attribute) attr)) ; we should check ?

;;;
;;; This way.  (Naming of `with-' is a idea of pascal :-)
;;; 
(defmacro with-buffer (buffer &rest body)
  "Set current buffer to BUFFER and execute a BODY. And back to previous
buffer."
  (append (list 'save-excursion (list 'set-buffer buffer)) body))


;;;
;;; print each buffer.
;;;
(defun list-buffer (buf filep non-internalp)
  "Print a BUFFER's information to standard-output."
  (let* ((col1 19) (col2 25) (col3 40) (minspace 1)
	 (attr (buffer-attribute buf))
	 (file (access-buffer-attribute attr 'buffer-file-name))
	 ;;(file (with-buffer buf buffer-file-name))
	 (name (buffer-name buf)) )
    (if (or (and filep (not file))
	    (and non-internalp (and (stringp name) (= ?\  (aref name 0)))))
	nil
      ;;| MR |
      (princ (if (eq this-buffer buf) "." " " )) ; `this-buffer' is dynamically
						 ; bound.  Is it better to pass
						 ; as an argument ? (yappari?)
      (princ (if (buffer-modified-p buf) "*" " "))
      (princ (if (access-buffer-attribute attr 'buffer-read-only) "%" " "))
      ;; (princ (if (with-buffer buf buffer-read-only) "%" " "))
      (princ " ")
      ;; buffer-name
      (princ name)
      (indent-to col1 2)
      ;; 
      (princ (access-buffer-attribute attr 'buffer-size))
      ;;(princ (with-buffer buf (buffer-size)))
      (indent-to col2 minspace)
      ;;
      (princ (access-buffer-attribute attr 'mode-name))
      ;;(princ (with-buffer buf mode-name))
      (indent-to col3 minspace)
      ;;
      (and file (princ file))
      (terpri) )))

;;;
;;; user interface.
;;;
(defun list-all-buffers (arg)
  "Display a list of names of existing buffers.
Negative argument means show all buffers (i.e. buffer which name is start
with space also displayed.
Other prefix argument means show file associated buffer only."
  (interactive "P")
  (let* ((this-buffer (current-buffer))
	 ;; Except for M--, it is fast to use original functoin `list-buffers'
	 (func (cond ((consp arg)
		      (function (lambda (buf) (list-buffer buf t nil))) )
		     ((eq '- arg)
		      (function (lambda (buf) (list-buffer buf nil nil))) )
		     (t
		      (function (lambda (buf) (list-buffer buf nil t))) ))))
    (with-output-to-temp-buffer "*Buffer List*"
      (set-buffer standard-output)
      (Buffer-menu-mode)
      (buffer-flush-undo standard-output)
      (setq buffer-read-only nil)
      (princ "\
 MR Buffer         Size  Mode           File\n\
 -- ------         ----  ----           ----\n\
")
      (mapcar func (buffer-list))
      (setq buffer-read-only t) )))
